@@ -36,21 +36,21 @@ MAG_DBS_TO_ANNOTATE = ('kegg', 'kofam', 'kofam_ko_list', 'uniref', 'peptidase',
 
 def filter_fasta(fasta_loc, min_len=5000, output_loc=None):
     """Removes sequences shorter than a set minimum from fasta files, outputs an object or to a file"""
-    kept_seqs = (seq for seq in read_sequence(fasta_loc, format='fasta') if len(seq) > min_len)
+    kept_seqs = (seq for seq in read_sequence(fasta_loc, format='fasta') if len(seq) >= min_len)
     if output_loc is None:
         return list(kept_seqs)
     else:
         write_sequence(kept_seqs, format='fasta', into=output_loc)
 
 
-def run_prodigal(fasta_loc, output_dir, verbose=False):
+def run_prodigal(fasta_loc, output_dir, mode='meta', trans_table='11', verbose=False):
     """Runs the prodigal gene caller on a given fasta file, outputs resulting files to given directory"""
     output_gff = path.join(output_dir, 'genes.gff')
     output_fna = path.join(output_dir, 'genes.fna')
     output_faa = path.join(output_dir, 'genes.faa')
 
-    run_process(['prodigal', '-i', fasta_loc, '-p', 'meta', '-f', 'gff', '-o', output_gff, '-a', output_faa, '-d',
-                 output_fna], verbose=verbose)
+    run_process(['prodigal', '-i', fasta_loc, '-p', mode, '-g', trans_table, '-f', 'gff', '-o', output_gff, '-a',
+                 output_faa, '-d', output_fna], verbose=verbose)
     return output_gff, output_fna, output_faa
 
 
@@ -250,13 +250,16 @@ def run_hmmscan_kofam(gene_faa, kofam_hmm, output_dir, ko_list, threads=1, verbo
                 raise ValueError(ko_row['score_type'])
             frame = frame.loc[score.astype(float) > float(ko_row.threshold)]
             is_sig.append(frame)
-        ko_hits_sig = pd.concat(is_sig)
+        if len(is_sig) == 0:
+            return pd.DataFrame(columns=['kegg_id', 'kegg_hit'])
+        else:
+            ko_hits_sig = pd.concat(is_sig)
 
-        kegg_dict = dict()
-        for gene, frame in ko_hits_sig.groupby('query_id'):
-            kegg_dict[gene] = [','.join([i for i in frame.target_id]),
-                               '; '.join([ko_list.loc[i, 'definition'] for i in frame.target_id])]
-        return pd.DataFrame(kegg_dict, index=['kegg_id', 'kegg_hit']).transpose()
+            kegg_dict = dict()
+            for gene, frame in ko_hits_sig.groupby('query_id'):
+                kegg_dict[gene] = [','.join([i for i in frame.target_id]),
+                                   '; '.join([ko_list.loc[i, 'definition'] for i in frame.target_id])]
+            return pd.DataFrame(kegg_dict, index=['kegg_id', 'kegg_hit']).transpose()
     else:
         return pd.DataFrame(columns=['kegg_id', 'kegg_hit'])
 
@@ -379,7 +382,8 @@ def assign_grades(annotations):
             rank = 'B'
         elif not pd.isna(row.get('kegg_hit')) or not pd.isna(row.get('uniref_hit')):
             rank = 'C'
-        elif not pd.isna(row.get('pfam_hits')):
+        elif not pd.isna(row.get('pfam_hits')) or not pd.isna(row.get('cazy_hits'))\
+                or not pd.isna(row.get('peptidase_hit')):
             rank = 'D'
         else:
             rank = 'E'
@@ -495,7 +499,8 @@ def make_gbk_from_gff_and_fasta(gff_loc='genes.gff', fasta_loc='scaffolds.fna',
     # now we can only ready one record from the gff/fasta hybrid at a time
     # read a record at a time till end of the fasta
     genbank_records = ''
-    for i in range(1, capture_fasta.getvalue().count('>') + 1):
+    # +1 because there is no \n> for the first line in the file and +1 because we are indexing starting at 1
+    for i in range(1, capture_fasta.getvalue().count('\n>') + 1 + 1):
         seq = read_sequence(io.StringIO(concat_gff), format='gff3', into=Sequence, seq_num=i)
         seq.metadata['LOCUS'] = {'locus_name': seq.metadata['id'],
                                  'size': len(seq),
@@ -607,7 +612,10 @@ def add_intervals_to_gff(annotations_loc, gff_loc, len_dict, interval_function,
     # process trnas to intervals
     annotation_dict = dict()
     for scaffold, frame in annotation_frame.groupby(groupby_column):
-        scaffold = scaffold.strip()
+        if type(scaffold) is not str:
+            scaffold = str(scaffold)
+        else:
+            scaffold = scaffold.strip()
         im = IntervalMetadata(len_dict[scaffold])
         for i, (_, row) in enumerate(frame.iterrows()):
             i += 1
@@ -791,9 +799,10 @@ def annotate_orfs(gene_faa, db_locs, tmp_dir, start_time, db_handler, custom_db_
     return annotations
 
 
-def annotate_fasta(fasta_loc, fasta_name, output_dir, db_locs, db_handler, min_contig_size=5000, custom_db_locs=(),
-                   bit_score_threshold=60, rbh_bit_score_threshold=350, skip_trnascan=False, start_time=datetime.now(),
-                   threads=1, rename_bins=True, keep_tmp_dir=False, verbose=False):
+def annotate_fasta(fasta_loc, fasta_name, output_dir, db_locs, db_handler, min_contig_size=5000, prodigal_mode='meta',
+                   trans_table='11', custom_db_locs=(), bit_score_threshold=60, rbh_bit_score_threshold=350,
+                   skip_trnascan=False, start_time=datetime.now(), threads=1, rename_bins=True, keep_tmp_dir=False,
+                   verbose=False):
     """Annotated a single multifasta file, all file based outputs will be in output_dir"""
     # make temporary directory
     tmp_dir = path.join(output_dir, 'tmp')
@@ -805,10 +814,11 @@ def annotate_fasta(fasta_loc, fasta_name, output_dir, db_locs, db_handler, min_c
 
     if stat(filtered_fasta).st_size == 0:
         warnings.warn('No sequences were longer than min_contig_size')
-        return pd.DataFrame()
+        return None
 
     # predict ORFs with prodigal
-    gene_gff, gene_fna, gene_faa = run_prodigal(filtered_fasta, tmp_dir, verbose=verbose)
+    gene_gff, gene_fna, gene_faa = run_prodigal(filtered_fasta, tmp_dir, mode=prodigal_mode, trans_table=trans_table,
+                                                verbose=verbose)
 
     # annotate ORFs
     annotations = annotate_orfs(gene_faa, db_locs, tmp_dir, start_time, db_handler, custom_db_locs, bit_score_threshold,
@@ -834,6 +844,13 @@ def annotate_fasta(fasta_loc, fasta_name, output_dir, db_locs, db_handler, min_c
     renamed_gffs = path.join(output_dir, 'genes.annotated.gff')
     annotate_gff(gene_gff, renamed_gffs, annotations, prefix=prefix)
 
+    # add fasta name to frame and index, append to list
+    annotations.insert(0, 'fasta', fasta_name)
+    if rename_bins:
+        annotations.index = annotations.fasta + '_' + annotations.index
+    annotations_loc = path.join(output_dir, 'annotations.tsv')
+    annotations.to_csv(annotations_loc, sep='\t')
+
     # get tRNAs and rRNAs
     len_dict = {i.metadata['id']: len(i) for i in read_sequence(renamed_scaffolds, format='fasta')}
     if not skip_trnascan:
@@ -862,13 +879,6 @@ def annotate_fasta(fasta_loc, fasta_name, output_dir, db_locs, db_handler, min_c
     if not keep_tmp_dir:
         rmtree(tmp_dir)
 
-    # add fasta name to frame and index, append to list
-    annotations.insert(0, 'fasta', fasta_name)
-    if rename_bins:
-        annotations.index = annotations.fasta + '_' + annotations.index
-    annotations_loc = path.join(output_dir, 'annotations.tsv')
-    annotations.to_csv(annotations_loc, sep='\t')
-
     return Annotation(name=fasta_name, scaffolds=renamed_scaffolds, genes_faa=annotated_faa, genes_fna=annotated_fna,
                       gff=renamed_gffs, gbk=current_gbk, annotations=annotations_loc, trnas=trna_loc, rrnas=rrna_loc)
 
@@ -889,9 +899,10 @@ def filter_db_locs(db_locs, low_mem_mode=False, use_uniref=False, master_list=MA
     return db_locs
 
 
-def annotate_fastas(fasta_locs, output_dir, db_locs, db_handler, min_contig_size=5000, bit_score_threshold=60,
-                    rbh_bit_score_threshold=350, custom_db_name=(), custom_fasta_loc=(), skip_trnascan=False,
-                    rename_bins=True, keep_tmp_dir=True, start_time=datetime.now(), threads=10, verbose=True):
+def annotate_fastas(fasta_locs, output_dir, db_locs, db_handler, min_contig_size=5000, prodigal_mode='meta',
+                    trans_table='11', bit_score_threshold=60, rbh_bit_score_threshold=350, custom_db_name=(),
+                    custom_fasta_loc=(), skip_trnascan=False, rename_bins=True, keep_tmp_dir=True,
+                    start_time=datetime.now(), threads=10, verbose=True):
     # check for no conflicting options/configurations
     tmp_dir = path.join(output_dir, 'working_dir')
     mkdir(tmp_dir)
@@ -910,23 +921,24 @@ def annotate_fastas(fasta_locs, output_dir, db_locs, db_handler, min_contig_size
         fasta_dir = path.join(tmp_dir, fasta_name)
         mkdir(fasta_dir)
         annotations_list.append(annotate_fasta(fasta_loc, fasta_name, fasta_dir, db_locs, db_handler, min_contig_size,
-                                               custom_db_locs, bit_score_threshold, rbh_bit_score_threshold,
-                                               skip_trnascan, start_time, threads, rename_bins, keep_tmp_dir, verbose))
+                                               prodigal_mode, trans_table, custom_db_locs, bit_score_threshold,
+                                               rbh_bit_score_threshold, skip_trnascan, start_time, threads, rename_bins,
+                                               keep_tmp_dir, verbose))
     print('%s: Annotations complete, processing annotations' % str(datetime.now() - start_time))
 
     # merge annotation dicts
-    all_annotations = pd.concat([i.get_annotations() for i in annotations_list], sort=False)
+    all_annotations = pd.concat([i.get_annotations() for i in annotations_list if i is not None], sort=False)
     all_annotations = all_annotations.sort_values(['fasta', 'scaffold', 'gene_position'])
 
     # merge gene files
-    merge_files([i.genes_fna_loc for i in annotations_list], path.join(output_dir, 'genes.fna'))
-    merge_files([i.genes_faa_loc for i in annotations_list], path.join(output_dir, 'genes.faa'))
-    merge_files([i.scaffolds_loc for i in annotations_list], path.join(output_dir, 'scaffolds.fna'))
-    merge_files([i.gff_loc for i in annotations_list], path.join(output_dir, 'genes.gff'), True)
-    trnas_locs = [i.trnas_loc for i in annotations_list if i.trnas_loc is not None]
+    merge_files([i.genes_fna_loc for i in annotations_list if i is not None], path.join(output_dir, 'genes.fna'))
+    merge_files([i.genes_faa_loc for i in annotations_list if i is not None], path.join(output_dir, 'genes.faa'))
+    merge_files([i.scaffolds_loc for i in annotations_list if i is not None], path.join(output_dir, 'scaffolds.fna'))
+    merge_files([i.gff_loc for i in annotations_list if i is not None], path.join(output_dir, 'genes.gff'), True)
+    trnas_locs = [i.trnas_loc for i in annotations_list if i is not None if i.trnas_loc is not None]
     if len(trnas_locs) > 0:
         merge_files(trnas_locs, path.join(output_dir, 'trnas.tsv'), True)
-    rrnas_locs = [i.rrnas_loc for i in annotations_list if i.rrnas_loc is not None]
+    rrnas_locs = [i.rrnas_loc for i in annotations_list if i is not None if i.rrnas_loc is not None]
     if len(rrnas_locs) > 0:
         merge_files(rrnas_locs, path.join(output_dir, 'rrnas.tsv'), True)
 
@@ -934,7 +946,8 @@ def annotate_fastas(fasta_locs, output_dir, db_locs, db_handler, min_contig_size
     gbk_dir = path.join(output_dir, 'genbank')
     mkdir(gbk_dir)
     for anno in annotations_list:
-        copy2(anno.gbk_loc, path.join(gbk_dir, '%s.gbk' % anno.name))
+        if anno is not None:
+            copy2(anno.gbk_loc, path.join(gbk_dir, '%s.gbk' % anno.name))
 
     # clean up
     if not keep_tmp_dir:
@@ -942,30 +955,46 @@ def annotate_fastas(fasta_locs, output_dir, db_locs, db_handler, min_contig_size
     return all_annotations
 
 
-def annotate_bins_cmd(input_fasta, output_dir='.', min_contig_size=5000, bit_score_threshold=60,
-                  rbh_bit_score_threshold=350, custom_db_name=(), custom_fasta_loc=(), use_uniref=False,
-                  skip_trnascan=False, gtdb_taxonomy=(), checkm_quality=(), keep_tmp_dir=True,
-                  low_mem_mode=False, threads=10, verbose=True):
+def annotate_bins_cmd(input_fasta, output_dir='.', min_contig_size=5000, prodigal_mode='meta', trans_table='11',
+                      bit_score_threshold=60, rbh_bit_score_threshold=350, custom_db_name=(), custom_fasta_loc=(),
+                      use_uniref=False, skip_trnascan=False, gtdb_taxonomy=(), checkm_quality=(), keep_tmp_dir=True,
+                      low_mem_mode=False, threads=10, verbose=True):
     fasta_locs = glob(input_fasta)
     if len(fasta_locs) == 0:
-        raise ValueError('Given fasta locations returns no paths: %s')
+        raise ValueError('Given fasta locations returns no paths: %s' % input_fasta)
     print('%s fastas found' % len(fasta_locs))
     rename_bins = True
-    annotate_bins(fasta_locs, output_dir, min_contig_size, bit_score_threshold, rbh_bit_score_threshold, custom_db_name,
-                  custom_fasta_loc, use_uniref, skip_trnascan, gtdb_taxonomy, checkm_quality, rename_bins, keep_tmp_dir,
-                  low_mem_mode, threads, verbose)
+    annotate_bins(fasta_locs, output_dir, min_contig_size, prodigal_mode, trans_table, bit_score_threshold,
+                  rbh_bit_score_threshold, custom_db_name, custom_fasta_loc, use_uniref, skip_trnascan, gtdb_taxonomy,
+                  checkm_quality, rename_bins, keep_tmp_dir, low_mem_mode, threads, verbose)
 
 
 # TODO: Add force flag to remove output dir if it already exists
 # TODO: Add continute flag to continue if output directory already exists
-def annotate_bins(fasta_locs, output_dir='.', min_contig_size=2500, bit_score_threshold=60,
-                  rbh_bit_score_threshold=350, custom_db_name=(), custom_fasta_loc=(), use_uniref=False,
-                  skip_trnascan=False, gtdb_taxonomy=(), checkm_quality=(), rename_bins=True, keep_tmp_dir=True,
-                  low_mem_mode=False, threads=10, verbose=True):
+# TODO: make fasta loc either a string or list to remove annotate_bins_cmd and annotate_called_genes_cmd?
+def annotate_bins(fasta_locs, output_dir='.', min_contig_size=2500, prodigal_mode='meta', trans_table='11',
+                  bit_score_threshold=60, rbh_bit_score_threshold=350, custom_db_name=(), custom_fasta_loc=(),
+                  use_uniref=False, skip_trnascan=False, gtdb_taxonomy=(), checkm_quality=(), rename_bins=True,
+                  keep_tmp_dir=True, low_mem_mode=False, threads=10, verbose=True):
     # set up
     start_time = datetime.now()
     print('%s: Annotation started' % str(datetime.now()))
 
+    # check inputs
+    prodigal_modes = ['train', 'meta', 'single']
+    if prodigal_mode not in prodigal_modes:
+        raise ValueError('Prodigal mode must be one of %s.' % ', '.join(prodigal_modes))
+    elif prodigal_mode in ['normal', 'single']:
+        warnings.warn('When running prodigal in single mode your bins must have long contigs (average length >3 Kbp), '
+                      'be long enough (total length > 500 Kbp) and have very low contamination in order for prodigal '
+                      'training to work well.')
+
+    # prodigal_trans_tables = ['auto'] + [str(i) for i in range(1, 26)]
+    prodigal_trans_tables = [str(i) for i in range(1, 26)]
+    if not trans_table in prodigal_trans_tables:
+        # raise ValueError('Prodigal translation table must be 1-25 or auto')
+        raise ValueError('Prodigal translation table must be 1-25')
+
     # get database locations
     db_locs = get_database_locs()
     db_handler = DatabaseHandler(db_locs['description_db'])
@@ -973,25 +1002,64 @@ def annotate_bins(fasta_locs, output_dir='.', min_contig_size=2500, bit_score_th
 
     mkdir(output_dir)
 
-    all_annotations = annotate_fastas(fasta_locs, output_dir, db_locs, db_handler, min_contig_size, bit_score_threshold,
-                                      rbh_bit_score_threshold, custom_db_name, custom_fasta_loc, skip_trnascan,
-                                      rename_bins, keep_tmp_dir, start_time, threads, verbose)
+    all_annotations = annotate_fastas(fasta_locs, output_dir, db_locs, db_handler, min_contig_size, prodigal_mode,
+                                      trans_table, bit_score_threshold, rbh_bit_score_threshold, custom_db_name,
+                                      custom_fasta_loc, skip_trnascan, rename_bins, keep_tmp_dir, start_time, threads,
+                                      verbose)
     # if given add taxonomy information
     if len(gtdb_taxonomy) > 0:
         gtdb_taxonomy = pd.concat([pd.read_csv(i, sep='\t', index_col=0) for i in gtdb_taxonomy])
-        all_annotations['bin_taxonomy'] = [gtdb_taxonomy.loc[i, 'classification'] for i in all_annotations.fasta]
+        taxonomy = list()
+        taxonomy_missing_bins = list()
+        for i in all_annotations.fasta:
+            # add taxonomy
+            if i in gtdb_taxonomy.index:
+                taxonomy.append(gtdb_taxonomy.loc[i, 'classification'])
+            else:
+                taxonomy.append(i)
+                taxonomy_missing_bins.append(i)
+        for i in set(taxonomy_missing_bins):
+            warnings.warn('Bin %s was not found in taxonomy file, replaced with bin name.' % i)
+        all_annotations['bin_taxonomy'] = taxonomy
+    # if given add quality information
     if len(checkm_quality) > 0:
         checkm_quality = pd.concat([pd.read_csv(i, sep='\t', index_col=0) for i in checkm_quality])
         checkm_quality.index = [strip_endings(i, ['.fa', '.fasta', '.fna']) for i in checkm_quality.index]
-        all_annotations['bin_completeness'] = [checkm_quality.loc[i, 'Completeness'] for i in all_annotations.fasta]
-        all_annotations['bin_contamination'] = [checkm_quality.loc[i, 'Contamination']
-                                                for i in all_annotations.fasta]
+
+        completeness = list()
+        contamination = list()
+        quality_missing_bins = list()
+        for i in all_annotations.fasta:
+            # add completeness and contamination
+            if i in checkm_quality.index:
+                completeness.append(checkm_quality.loc[i, 'Completeness'])
+                contamination.append(checkm_quality.loc[i, 'Contamination'])
+            else:
+                completeness.append(0)
+                contamination.append(100)
+                quality_missing_bins.append(i)
+            for i in set(quality_missing_bins):
+                warnings.warn('Bin %s was not found in quality file, '
+                              'replaced with completeness 0 and contamination 100.' % i)
+        all_annotations['bin_completeness'] = completeness
+        all_annotations['bin_contamination'] = contamination
     all_annotations.to_csv(path.join(output_dir, 'annotations.tsv'), sep='\t')
 
     print("%s: Completed annotations" % str(datetime.now() - start_time))
 
 
-def annotate_called_genes(input_faa, output_dir='.', bit_score_threshold=60, rbh_bit_score_threshold=350,
+def annotate_called_genes_cmd(input_faa, output_dir='.', bit_score_threshold=60, rbh_bit_score_threshold=350,
+                              custom_db_name=(), custom_fasta_loc=(), use_uniref=False, keep_tmp_dir=True,
+                              low_mem_mode=False, threads=10, verbose=True):
+    fasta_locs = glob(input_faa)
+    if len(fasta_locs) == 0:
+        raise ValueError('Given fasta locations returns no paths: %s' % input_faa)
+    print('%s fastas found' % len(fasta_locs))
+    annotate_called_genes(fasta_locs, output_dir, bit_score_threshold, rbh_bit_score_threshold, custom_db_name,
+                          custom_fasta_loc, use_uniref, keep_tmp_dir, low_mem_mode, threads, verbose)
+
+
+def annotate_called_genes(fasta_locs, output_dir='.', bit_score_threshold=60, rbh_bit_score_threshold=350,
                           custom_db_name=(), custom_fasta_loc=(), use_uniref=False, keep_tmp_dir=True,
                           low_mem_mode=False, threads=10, verbose=True):
     # set up
@@ -1015,9 +1083,9 @@ def annotate_called_genes(input_faa, output_dir='.', bit_score_threshold=60, rbh
     # annotate
     annotation_locs = list()
     faa_locs = list()
-    for fasta_loc in glob(input_faa):
+    for fasta_loc in fasta_locs:
         # set up
-        fasta_name = path.splitext(path.basename(remove_suffix(input_faa, '.gz')))[0]
+        fasta_name = path.splitext(path.basename(remove_suffix(fasta_loc, '.gz')))[0]
         fasta_dir = path.join(tmp_dir, fasta_name)
         mkdir(fasta_dir)
 
@@ -1026,7 +1094,7 @@ def annotate_called_genes(input_faa, output_dir='.', bit_score_threshold=60, rbh
                                     bit_score_threshold, rbh_bit_score_threshold, threads, verbose)
 
         annotated_faa = path.join(fasta_dir, 'genes.faa')
-        create_annotated_fasta(input_faa, annotations, annotated_faa, name=fasta_name)
+        create_annotated_fasta(fasta_loc, annotations, annotated_faa, name=fasta_name)
         faa_locs.append(annotated_faa)
 
         # add fasta name to frame and index, write file
