@@ -1,6 +1,7 @@
 from datetime import datetime
 from os import path, mkdir
 import re
+import warnings
 
 import pandas as pd
 import numpy as np
@@ -59,6 +60,7 @@ def remove_bad_chars_fasta(fasta):
 def remove_bad_chars_virsorter_affi_contigs(virsorter_in):
     new_lines = list()
     for line in open(virsorter_in).readlines():
+        line = line.strip()
         if line.startswith('>'):
             line = line.lstrip('>')
             split_line = line.split('|')
@@ -68,7 +70,7 @@ def remove_bad_chars_virsorter_affi_contigs(virsorter_in):
             split_line = line.split('|')
             split_line[0] = split_line[0].replace(';', '_').replace('=', '_')
             new_lines.append('|'.join(split_line))
-    return ''.join(new_lines)
+    return '%s\n' % '\n'.join(new_lines)
 
 
 def remove_bad_chars(input_fasta=None, input_virsorter_affi_contigs=None, output=None):
@@ -238,7 +240,7 @@ def get_metabolic_flags(annotations, metabolic_genes, amgs, verified_amgs, scaff
         for gene, row in scaffold_annotations.iterrows():
             # set up
             flags = ''
-            gene_annotations = set(get_ids_from_annotation(pd.DataFrame(row).transpose()).keys())
+            gene_annotations = set(get_ids_from_annotation(pd.DataFrame(row).transpose()).keys())  # TODO: Fix
             # is viral
             if not pd.isna(row['vogdb_categories']):
                 if len({'Xr', 'Xs'} & set(row['vogdb_categories'].split(';'))) > 0:
@@ -308,50 +310,7 @@ def get_virsorter_affi_contigs_name(scaffold):
     return virsorter_scaffold_name
 
 
-def annotate_vgfs(input_fasta, virsorter_affi_contigs=None, output_dir='.', min_contig_size=2500,
-                  bit_score_threshold=60, rbh_bit_score_threshold=350, custom_db_name=(), custom_fasta_loc=(),
-                  use_uniref=False, low_mem_mode=False, skip_trnascan=False, keep_tmp_dir=True,
-                  threads=10, verbose=True):
-    # set up
-    start_time = datetime.now()
-    print('%s: Viral annotation started' % str(datetime.now()))
-
-    # get database locations
-    db_locs = get_database_locs()
-    db_handler = DatabaseHandler(db_locs['description_db'])
-    db_locs_anno = filter_db_locs(db_locs, low_mem_mode, use_uniref, VMAG_DBS_TO_ANNOTATE)
-
-    if virsorter_affi_contigs is not None:
-        virsorter_hits = get_virsorter_hits(virsorter_affi_contigs)
-    else:
-        virsorter_hits = None
-
-    # split sequences into seperate fastas
-    mkdir(output_dir)
-    contig_dir = path.join(output_dir, 'vMAGs')
-    mkdir(contig_dir)
-    contig_locs = list()
-    for seq in read_sequence(input_fasta, format='fasta'):
-        if len(seq) >= min_contig_size:
-            if '=' in seq.metadata['id'] or ';' in seq.metadata['id']:
-                raise ValueError('FASTA headers must not have = or ; before the first space (%s). To run DRAM-v you '
-                                 'must rerun VIRSorter with = and ; removed from the headers or run DRAM-v.py '
-                                 'remove_bad_characters and then rerun DRAM-v' % seq.metadata['id'])
-            if virsorter_hits is not None:
-                if get_virsorter_affi_contigs_name(seq.metadata['id']) not in virsorter_hits['name'].values:
-                    raise ValueError("No virsorter calls found in %s for scaffold %s from input fasta" %
-                                     (virsorter_affi_contigs, seq.metadata['id']))
-            contig_loc = path.join(contig_dir, '%s.fasta' % seq.metadata['id'])
-            write_sequence((i for i in [seq]), format='fasta', into=contig_loc)
-            contig_locs.append(contig_loc)
-
-    # annotate vMAGs
-    rename_bins = False
-    annotations = annotate_fastas(contig_locs, output_dir, db_locs_anno, db_handler, min_contig_size,
-                                  bit_score_threshold, rbh_bit_score_threshold, custom_db_name, custom_fasta_loc,
-                                  skip_trnascan, rename_bins, keep_tmp_dir, start_time, threads, verbose)
-    print('%s: Annotations complete, processing annotations' % str(datetime.now() - start_time))
-
+def add_dramv_scores_and_flags(annotations, db_locs=None, virsorter_hits=None, input_fasta=None):
     # setting up scoring viral genes
     amg_database_frame = pd.read_csv(db_locs['amg_database'], sep='\t')
     genome_summary_form = pd.read_csv(db_locs['genome_summary_form'], sep='\t', index_col=0)
@@ -385,11 +344,70 @@ def annotate_vgfs(input_fasta, virsorter_affi_contigs=None, output_dir='.', min_
                                                              verified_amgs, scaffold_length_dict))
 
     # downgrade B flag auxiliary scores
-    if virsorter_affi_contigs is not None:
+    if virsorter_hits is not None:
         annotations['auxiliary_score'] = pd.Series({gene: (4 if 'B' in row['amg_flags'] and row['auxiliary_score'] < 4
                                                            else row['auxiliary_score'])
                                                     for gene, row in annotations.iterrows()})
 
+    return annotations
+
+
+def annotate_vgfs(input_fasta, virsorter_affi_contigs=None, output_dir='.', min_contig_size=2500, prodigal_mode='meta',
+                  trans_table='11', bit_score_threshold=60, rbh_bit_score_threshold=350, custom_db_name=(),
+                  custom_fasta_loc=(), use_uniref=False, low_mem_mode=False, skip_trnascan=False, keep_tmp_dir=True,
+                  threads=10, verbose=True):
+    # set up
+    start_time = datetime.now()
+    print('%s: Viral annotation started' % str(datetime.now()))
+
+    # check inputs
+    prodigal_modes = ['train', 'meta', 'single']
+    if prodigal_mode not in prodigal_modes:
+        raise ValueError('Prodigal mode must be one of %s.' % ', '.join(prodigal_modes))
+    elif prodigal_mode in ['normal', 'single']:
+        warnings.warn('When running prodigal in single mode your bins must have long contigs (average length >3 Kbp), '
+                      'be long enough (total length > 500 Kbp) and have very low contamination in order for prodigal '
+                      'training to work well.')
+
+    # get database locations
+    db_locs = get_database_locs()
+    db_handler = DatabaseHandler(db_locs['description_db'])
+    db_locs_anno = filter_db_locs(db_locs, low_mem_mode, use_uniref, VMAG_DBS_TO_ANNOTATE)
+
+    if virsorter_affi_contigs is not None:
+        virsorter_hits = get_virsorter_hits(virsorter_affi_contigs)
+    else:
+        virsorter_hits = None
+
+    # split sequences into seperate fastas
+    mkdir(output_dir)
+    contig_dir = path.join(output_dir, 'vMAGs')
+    mkdir(contig_dir)
+    contig_locs = list()
+    for seq in read_sequence(input_fasta, format='fasta'):
+        if len(seq) >= min_contig_size:
+            if '=' in seq.metadata['id'] or ';' in seq.metadata['id']:
+                raise ValueError('FASTA headers must not have = or ; before the first space (%s). To run DRAM-v you '
+                                 'must rerun VIRSorter with = and ; removed from the headers or run DRAM-v.py '
+                                 'remove_bad_characters and then rerun DRAM-v' % seq.metadata['id'])
+            if virsorter_hits is not None:
+                if get_virsorter_affi_contigs_name(seq.metadata['id']) not in virsorter_hits['name'].values:
+                    raise ValueError("No virsorter calls found in %s for scaffold %s from input fasta" %
+                                     (virsorter_affi_contigs, seq.metadata['id']))
+            contig_loc = path.join(contig_dir, '%s.fasta' % seq.metadata['id'])
+            write_sequence((i for i in [seq]), format='fasta', into=contig_loc)
+            contig_locs.append(contig_loc)
+
+    # annotate vMAGs
+    rename_bins = False
+    annotations = annotate_fastas(contig_locs, output_dir, db_locs_anno, db_handler, min_contig_size, prodigal_mode,
+                                  trans_table, bit_score_threshold, rbh_bit_score_threshold, custom_db_name,
+                                  custom_fasta_loc, skip_trnascan, rename_bins, keep_tmp_dir, start_time, threads,
+                                  verbose)
+    print('%s: Annotations complete, assigning auxiliary scores and flags' % str(datetime.now() - start_time))
+
+    annotations = add_dramv_scores_and_flags(annotations, db_locs, virsorter_hits, input_fasta)
+
     # write annotations
     annotations.to_csv(path.join(output_dir, 'annotations.tsv'), sep='\t')
 
