@@ -163,16 +163,20 @@ def process_pfam_descriptions(pfam_hmm_dat):
     return description_list
 
 
-def download_and_process_dbcan(dbcan_hmm=None, output_dir='.', dbcan_release='8', verbose=True):
+def download_and_process_dbcan(dbcan_hmm=None, output_dir='.', dbcan_release='9', verbose=True):
     if dbcan_hmm is None:  # download database if not provided
         dbcan_hmm = path.join(output_dir, 'dbCAN-HMMdb-V%s.txt' % dbcan_release)
-        download_file('http://bcb.unl.edu/dbCAN2/download/Databases/dbCAN-HMMdb-V%s.txt' % dbcan_release, dbcan_hmm,
-                      verbose=verbose)
+        if int(dbcan_release) < 9:
+            download_file('http://bcb.unl.edu/dbCAN2/download/Databases/dbCAN-HMMdb-V%s.txt' % dbcan_release, dbcan_hmm,
+                          verbose=verbose)
+        else:
+            download_file('http://bcb.unl.edu/dbCAN2/download/dbCAN-HMMdb-V%s.txt' % dbcan_release, dbcan_hmm,
+                          verbose=verbose)
     run_process(['hmmpress', '-f', dbcan_hmm], verbose=verbose)
     return dbcan_hmm
 
 
-def download_dbcan_descriptions(output_dir='.', upload_date='07312019', verbose=True):
+def download_dbcan_descriptions(output_dir='.', upload_date='07302020', verbose=True):
     dbcan_fam_activities = path.join(output_dir, 'CAZyDB.%s.fam-activities.txt' % upload_date)
     download_file('http://bcb.unl.edu/dbCAN2/download/Databases/CAZyDB.%s.fam-activities.txt' % upload_date,
                   dbcan_fam_activities, verbose=verbose)
@@ -342,17 +346,14 @@ def set_database_paths(kegg_db_loc=None, kofam_hmm_loc=None, kofam_ko_list_loc=N
     db_dict = check_exists_and_add_to_location_dict(etc_module_database_loc, 'etc_module_database', db_dict)
     db_dict = check_exists_and_add_to_location_dict(function_heatmap_form_loc, 'function_heatmap_form', db_dict)
     db_dict = check_exists_and_add_to_location_dict(amg_database_loc, 'amg_database', db_dict)
-
-    if description_db_loc is not None:
-        db_dict['description_db'] = description_db_loc
-    elif 'description_db' not in db_dict:
-        db_dict['description_db'] = None
     print('%s: Database locations added to CONFIG' % str(datetime.now() - start_time))
 
     if update_description_db:
-        populate_description_db(db_dict, start_time)
-        check_file_exists(description_db_loc)
+        if description_db_loc is None:
+            description_db_loc = db_dict['description_db']
+        populate_description_db(description_db_loc, db_dict, start_time)
         print('%s: Database descriptions updated' % str(datetime.now() - start_time))
+    db_dict = check_exists_and_add_to_location_dict(description_db_loc, 'description_db', db_dict)
 
     # change data paths
     if config_loc is None:
@@ -362,15 +363,21 @@ def set_database_paths(kegg_db_loc=None, kofam_hmm_loc=None, kofam_ko_list_loc=N
     print('%s: Database locations set' % str(datetime.now() - start_time))
 
 
-def populate_description_db(db_dict=None, start_time=None):
+def populate_description_db(output_loc=None, db_dict=None, start_time=None):
     if start_time is None:
         start_time = datetime.now()
         print('%s: Populating description database' % str(datetime.now() - start_time))
     # setup
     if db_dict is None:
         db_dict = get_database_locs()
-    if path.exists(db_dict['description_db']):
+
+    if db_dict.get('description_db') is None and output_loc is not None:
+        db_dict['description_db'] = output_loc
+    elif db_dict.get('description_db') is None and output_loc is None:
+        raise ValueError('Must provide output location if description db location is not set in configuration')
+    elif path.exists(db_dict['description_db']):
         remove(db_dict['description_db'])
+
     create_description_db(db_dict['description_db'])
     db_handler = DatabaseHandler(db_dict['description_db'])
     print('%s: Database connection established' % str(datetime.now() - start_time))
@@ -398,17 +405,24 @@ def populate_description_db(db_dict=None, start_time=None):
                           db_handler)
     print('%s: VOGdb descriptions added to description database' % str(datetime.now() - start_time))
     print('%s: Description database populated' % str(datetime.now() - start_time))
+    # TODO: if output loc is given then set in config file where the description database is
 
 
 def prepare_databases(output_dir, kegg_loc=None, gene_ko_link_loc=None, kofam_hmm_loc=None, kofam_ko_list_loc=None,
                       kegg_download_date=None, uniref_loc=None, uniref_version='90', pfam_loc=None, pfam_hmm_dat=None,
-                      dbcan_loc=None, dbcan_version='8', dbcan_fam_activities=None, dbcan_date='07312019',
+                      dbcan_loc=None, dbcan_version='9', dbcan_fam_activities=None, dbcan_date='07302020',
                       viral_loc=None, peptidase_loc=None, vogdb_loc=None, vogdb_version='latest', vog_annotations=None,
                       genome_summary_form_loc=None, module_step_form_loc=None, etc_module_database_loc=None,
                       function_heatmap_form_loc=None, amg_database_loc=None, skip_uniref=False,
                       keep_database_files=False, branch='master', threads=10, verbose=True):
     start_time = datetime.now()
     print('%s: Database preparation started' % str(datetime.now()))
+
+    # check inputs
+    if skip_uniref and uniref_loc is not None:
+        raise ValueError('Cannot skip UniRef processing and provide a location of UniRef. Skipping UniRef will cause '
+                         'provided UniRef file to not be used.')
+
     # check that all given files exist
     check_file_exists(kegg_loc)
     check_file_exists(gene_ko_link_loc)
