@@ -8,7 +8,7 @@ import re
 import numpy as np
 from datetime import datetime
 
-from mag_annotator.utils import get_database_locs, get_ids_from_annotation
+from mag_annotator.utils import get_database_locs, get_ids_from_annotation, get_ids_from_row
 
 # TODO: add RBH information to output
 # TODO: add flag to output table and not xlsx
@@ -23,7 +23,6 @@ HEATMAP_CELL_WIDTH = 10
 KO_REGEX = r'^K\d\d\d\d\d$'
 ETC_COVERAGE_COLUMNS = ['module_id', 'module_name', 'complex', 'genome', 'path_length', 'path_length_coverage',
                         'percent_coverage', 'genes', 'missing_genes', 'complex_module_name']
-GENOMES_PER_LIQUOR = 1000
 TAXONOMY_LEVELS = ['d', 'p', 'c', 'o', 'f', 'g', 's']
 
 
@@ -34,8 +33,8 @@ def get_ordered_uniques(seq):
 
 
 def fill_genome_summary_frame(annotations, genome_summary_frame, groupby_column):
+    genome_summary_id_sets = [set([k.strip() for k in j.split(',')]) for j in genome_summary_frame['gene_id']]
     for genome, frame in annotations.groupby(groupby_column, sort=False):
-        genome_summary_id_sets = [set([k.strip() for k in j.split(',')]) for j in genome_summary_frame['gene_id']]
         id_dict = get_ids_from_annotation(frame)
         counts = list()
         for i in genome_summary_id_sets:
@@ -48,6 +47,26 @@ def fill_genome_summary_frame(annotations, genome_summary_frame, groupby_column)
     return genome_summary_frame
 
 
+def fill_genome_summary_frame_gene_names(annotations, genome_summary_frame, groupby_column):
+    genome_summary_id_sets = [set([k.strip() for k in j.split(',')]) for j in genome_summary_frame['gene_id']]
+    for genome, frame in annotations.groupby(groupby_column, sort=False):
+        # make dict of identifiers to gene names
+        id_gene_dict = defaultdict(list)
+        for gene, row in frame.iterrows():
+            ids = get_ids_from_row(row)
+            for id_ in ids:
+                id_gene_dict[id_].append(gene)
+        # fill in genome summary_frame
+        values = list()
+        for id_set in genome_summary_id_sets:
+            this_value = list()
+            for id_ in id_set:
+                this_value += id_gene_dict[id_]
+            values.append(','.join(this_value))
+        genome_summary_frame[genome] = values
+    return genome_summary_frame
+
+
 def summarize_rrnas(rrnas_df, groupby_column='fasta'):
     genome_rrna_dict = dict()
     for genome, frame in rrnas_df.groupby(groupby_column):
@@ -161,8 +180,8 @@ def make_genome_stats(annotations, rrna_frame=None, trna_frame=None, groupby_col
                     row.append('')
                     has_rrna.append(False)
                 elif sixteens.shape[0] == 1:
-                    row.append('%s, (%s, %s)' % (sixteens[groupby_column].iloc[0], sixteens.begin.iloc[0],
-                                                 sixteens.end.iloc[0]))
+                    row.append('%s (%s, %s)' % (sixteens['scaffold'].iloc[0], sixteens.begin.iloc[0],
+                                                sixteens.end.iloc[0]))
                     has_rrna.append(True)
                 else:
                     row.append('%s present' % sixteens.shape[0])
@@ -537,7 +556,8 @@ def make_strings_no_repeats(genome_taxa_dict):
     return labels
 
 
-def summarize_genomes(input_file, trna_path=None, rrna_path=None, output_dir='.', groupby_column='fasta'):
+def summarize_genomes(input_file, trna_path=None, rrna_path=None, output_dir='.', groupby_column='fasta',
+                      custom_distillate=None, distillate_gene_names=False, genomes_per_product=1000):
     start_time = datetime.now()
 
     # read in data
@@ -565,6 +585,8 @@ def summarize_genomes(input_file, trna_path=None, rrna_path=None, output_dir='.'
 
     # read in dbs
     genome_summary_form = pd.read_csv(db_locs['genome_summary_form'], sep='\t')
+    if custom_distillate is not None:
+        genome_summary_form = pd.concat([genome_summary_form, pd.read_csv(custom_distillate, sep='\t')])
     genome_summary_form = genome_summary_form.drop('potential_amg', axis=1)
     module_steps_form = pd.read_csv(db_locs['module_step_form'], sep='\t')
     function_heatmap_form = pd.read_csv(db_locs['function_heatmap_form'], sep='\t')
@@ -575,13 +597,17 @@ def summarize_genomes(input_file, trna_path=None, rrna_path=None, output_dir='.'
     mkdir(output_dir)
 
     # make genome stats
-    genome_stats = make_genome_stats(annotations, rrna_frame, trna_frame)
+    genome_stats = make_genome_stats(annotations, rrna_frame, trna_frame, groupby_column=groupby_column)
     genome_stats.to_csv(path.join(output_dir, 'genome_stats.tsv'), sep='\t', index=None)
     print('%s: Calculated genome statistics' % (str(datetime.now() - start_time)))
 
     # make genome metabolism summary
     genome_summary = path.join(output_dir, 'metabolism_summary.xlsx')
-    summarized_genomes = make_genome_summary(annotations, genome_summary_form, trna_frame, rrna_frame, groupby_column)
+    if distillate_gene_names:
+        summarized_genomes = fill_genome_summary_frame_gene_names(annotations, genome_summary_form, groupby_column)
+    else:
+        summarized_genomes = make_genome_summary(annotations, genome_summary_form, trna_frame, rrna_frame,
+                                                 groupby_column)
     write_summarized_genomes_to_xlsx(summarized_genomes, genome_summary)
     print('%s: Generated genome metabolism summary' % (str(datetime.now() - start_time)))
 
@@ -604,12 +630,12 @@ def summarize_genomes(input_file, trna_path=None, rrna_path=None, output_dir='.'
     module_nets = {module: build_module_net(module_df)
                    for module, module_df in module_steps_form.groupby('module') if module in HEATMAP_MODULES}
 
-    if len(genome_order) > GENOMES_PER_LIQUOR:
+    if len(genome_order) > genomes_per_product:
         module_coverage_dfs = list()
         etc_coverage_dfs = list()
         function_dfs = list()
         # generates slice start and slice end to grab from genomes and labels from 0 to end of genome order
-        pairwise_iter = pairwise(list(range(0, len(genome_order), GENOMES_PER_LIQUOR)) + [len(genome_order) - 1])
+        pairwise_iter = pairwise(list(range(0, len(genome_order), genomes_per_product)) + [len(genome_order)])
         for i, (start, end) in enumerate(pairwise_iter):
             genomes = genome_order[start:end]
             annotations_subset = annotations.loc[[genome in genomes for genome in annotations[groupby_column]]]
@@ -628,7 +654,7 @@ def summarize_genomes(input_file, trna_path=None, rrna_path=None, output_dir='.'
         module_coverage_df, etc_coverage_df, function_df = fill_liquor_dfs(annotations, module_nets,
                                                                            etc_module_df,
                                                                            function_heatmap_form,
-                                                                           groupby_column='fasta')
+                                                                           groupby_column=groupby_column)
         liquor_df = make_liquor_df(module_coverage_df, etc_coverage_df, function_df)
         liquor_df.to_csv(path.join(output_dir, 'product.tsv'), sep='\t')
         liquor = make_liquor_heatmap(module_coverage_df, etc_coverage_df, function_df, genome_order, labels)
