@@ -7,7 +7,7 @@ from collections import defaultdict, Counter
 from datetime import datetime
 import warnings
 
-from mag_annotator.utils import get_database_locs
+from mag_annotator.utils import get_database_locs, get_ids_from_row, get_ids_from_annotation
 from mag_annotator.summarize_genomes import get_ordered_uniques
 
 VOGDB_TYPE_NAMES = {'Xr': 'Viral replication genes', 'Xs': 'Viral structure genes',
@@ -23,6 +23,23 @@ HEATMAP_CELL_WIDTH = 10
 
 defaultdict_list = partial(defaultdict, list)
 
+
+def add_custom_ms(annotations, distillate_form):
+    metabolic_genes = set(distillate_form.index)
+
+    new_amg_flags = list()
+    for gene, row in annotations.iterrows():
+        if 'M' in row['amg_flags']:
+            new_amg_flags.append(row['amg_flags'])
+        else:
+            gene_annotations = set(get_ids_from_annotation(pd.DataFrame(row).transpose()).keys())
+            if len(metabolic_genes & gene_annotations) > 0:
+                new_amg_flags.append(row['amg_flags'] + 'M')
+            else:
+                new_amg_flags.append(row['amg_flags'])
+    return new_amg_flags
+
+
 def filter_to_amgs(annotations, max_aux=4, remove_transposons=True, remove_fs=False):
     # def filter_to_amgs(annotations, max_aux=4, remove_transposons=True, remove_fs=False, remove_js=False):
     potential_amgs = list()
@@ -55,9 +72,14 @@ def make_viral_stats_table(annotations, potential_amgs, groupby_column='scaffold
     viral_stats_series = list()
     for scaffold, frame in annotations.groupby(groupby_column):
         # get virus information
-        virus_category = int(re.findall(r'-cat_\d$', scaffold)[0].split('_')[-1])  # viral category
+        virus_categories = re.findall(r'-cat_\d$', scaffold)
+        if len(virus_categories) > 0:
+            virus_category = int(virus_categories[0].split('_')[-1])  # viral category
+            virus_prophage = virus_category in [4, 5]  # virus is prophage
+        else:
+            virus_category = None
+            virus_prophage = None
         virus_circular = len(re.findall(r'-circular-cat_\d$', scaffold)) == 1  # virus is circular
-        virus_prophage = virus_category in [4, 5]  # virus is prophage
         virus_num_genes = len(frame)  # number of genes on viral contig
         virus_strand_switches = get_strand_switches(frame.strandedness)  # number of strand switches
         if scaffold in amg_counts:
@@ -73,30 +95,13 @@ def make_viral_stats_table(annotations, potential_amgs, groupby_column='scaffold
                                index=VIRUS_STATS_COLUMNS, name=scaffold)
         # get vogdb categories
         # when vogdb has multiple categories only the first is taken
-        gene_counts = Counter([i.split(';')[0] for i in frame.vogdb_categories.fillna('Xx')])
+        gene_counts = Counter([i.split(';')[0] for i in frame.vogdb_categories.replace('', 'Xx')])
         named_gene_counts = {VOGDB_TYPE_NAMES[key]: value for key, value in gene_counts.items()}
         gene_counts_series = pd.Series(named_gene_counts, name=scaffold)
         viral_stats_series.append(virus_data.append(gene_counts_series))
     return pd.DataFrame(viral_stats_series).fillna(0)
 
 
-def get_ids_from_row(row):
-    id_list = list()
-    # get kegg ids
-    if 'kegg_id' in row and not pd.isna(row['kegg_id']):
-        id_list += [j for j in row['kegg_id'].split(',')]
-    # get ec numbers
-    if 'kegg_hit' in row and not pd.isna(row['kegg_hit']):
-        id_list += [i[1:-1] for i in re.findall(r'\[EC:\d*.\d*.\d*.\d*\]', row['kegg_hit'])]
-    # get merops ids
-    if 'peptidase_family' in row and not pd.isna(row['peptidase_family']):
-        id_list += [j for j in row['peptidase_family'].split(';')]
-    # get cazy ids
-    if 'cazy_hits' in row and not pd.isna(row['cazy_hits']):
-        id_list += [j.strip().split(' ')[0] for j in row['cazy_hits'].split(';')]
-    return set(id_list)
-
-
 def make_viral_distillate(potential_amgs, genome_summary_frame):
     rows = list()
     for gene, row in potential_amgs.iterrows():
@@ -205,24 +210,29 @@ def make_viral_functional_heatmap(functional_df, vgf_order=None):
 
 
 # def summarize_vgfs(input_file, output_dir, groupby_column='scaffold', max_auxiliary_score=3, remove_transposons=False,
-#                    remove_fs=False, remove_js=False):
+#                    remove_fs=False, remove_js=False, custom_distillate=None):
 def summarize_vgfs(input_file, output_dir, groupby_column='scaffold', max_auxiliary_score=3,
-                   remove_transposons=False, remove_fs=False):
+                   remove_transposons=False, remove_fs=False, custom_distillate=None):
     start_time = datetime.now()
 
     # set up
-    annotations = pd.read_csv(input_file, sep='\t', index_col=0)
+    annotations = pd.read_csv(input_file, sep='\t', index_col=0).fillna('')
     db_locs = get_database_locs()
     if 'genome_summary_form' not in db_locs:
         raise ValueError('Genome summary form location must be set in order to summarize genomes')
     mkdir(output_dir)
     genome_summary_form = pd.read_csv(db_locs['genome_summary_form'], sep='\t', index_col=0)
+    if custom_distillate is not None:
+        custom_distillate_form = pd.read_csv(custom_distillate, sep='\t', index_col=0)
+        genome_summary_form = pd.concat([genome_summary_form, custom_distillate_form])
+        # add M's from custom distillate
+        annotations['amg_flags'] = add_custom_ms(annotations, custom_distillate_form)
     print('%s: Retrieved database locations and descriptions' % (str(datetime.now() - start_time)))
 
     # get potential AMGs
     # potential_amgs = filter_to_amgs(annotations.fillna(''), max_aux=max_auxiliary_score,
     #                                 remove_transposons=remove_transposons, remove_fs=remove_fs, remove_js=remove_js)
-    potential_amgs = filter_to_amgs(annotations.fillna(''), max_aux=max_auxiliary_score,
+    potential_amgs = filter_to_amgs(annotations, max_aux=max_auxiliary_score,
                                     remove_transposons=remove_transposons, remove_fs=remove_fs)
     print('%s: Determined potential amgs' % (str(datetime.now() - start_time)))
 
