@@ -57,11 +57,11 @@ if __name__ == '__main__':
                                     help="pfam hmm .dat file to get PF descriptions, if already downloaded "
                                          "(Pfam-A.hmm.dat.gz)")
     prepare_dbs_parser.add_argument('--dbcan_loc', default=None, help="File path to dbCAN, if already downloaded "
-                                                                      "(dbCAN-HMMdb-V8.txt)")
+                                                                      "(dbCAN-HMMdb-V9.txt)")
     prepare_dbs_parser.add_argument('--dbcan_fam_activities', default=None,
                                     help='CAZY family activities file, if already downloaded '
-                                         '(CAZyDB.07312019.fam-activities.txt)')
-    prepare_dbs_parser.add_argument('--dbcan_version', default='8', type=str, help='version of dbCAN to use')
+                                         '(CAZyDB.07302020.fam-activities.txt)')
+    prepare_dbs_parser.add_argument('--dbcan_version', default='9', type=str, help='version of dbCAN to use')
     prepare_dbs_parser.add_argument('--vogdb_loc', default=None,
                                     help='hmm file for vogdb, if already downloaded (vog.hmm.tar.gz)')
     prepare_dbs_parser.add_argument('--vog_annotations', default=None,
@@ -119,6 +119,8 @@ if __name__ == '__main__':
     set_db_locs_parser.set_defaults(func=set_database_paths)
 
     # parser for updating database descriptions
+    update_description_db_parser.add_argument('--output_loc', help="Location to store desciption database, will be "
+                                                                   "stored in location set in CONFIG if not given")
     update_description_db_parser.set_defaults(func=populate_description_db)
 
     # parser for updating DRAM databases only
