@@ -2,9 +2,9 @@
 
 import argparse
 
-from mag_annotator.annotate_bins import annotate_bins_cmd, annotate_called_genes
+from mag_annotator.annotate_bins import annotate_bins_cmd, annotate_called_genes_cmd
 from mag_annotator.summarize_genomes import summarize_genomes
-from mag_annotator.pull_sequences import pull_sequences
+from mag_annotator.pull_sequences import pull_sequences, get_gene_neighborhoods
 
 # TODO: refactor parses to limit duplication
 
@@ -21,14 +21,26 @@ if __name__ == '__main__':
                                            formatter_class=argparse.ArgumentDefaultsHelpFormatter)
     strainer_parser = subparsers.add_parser('strainer', help="Strain annotations down to genes of interest",
                                             formatter_class=argparse.ArgumentDefaultsHelpFormatter)
+    neighborhood_parser = subparsers.add_parser('neighborhoods', help="Find neighborhoods around genes of interest",
+                                                formatter_class=argparse.ArgumentDefaultsHelpFormatter)
 
     # parser for annotating mags, you know the real thing
+    # TODO: add don't rename flag and give warning that all contig names must be unique
     annotate_parser.add_argument('-i', '--input_fasta',
                                  help="fasta file, optionally with wildcards to point to multiple fastas",
                                  required=True)
     annotate_parser.add_argument('-o', '--output_dir', help="output directory")
     annotate_parser.add_argument('--min_contig_size', type=int, default=2500,
                                  help='minimum contig size to be used for gene prediction')
+    prodigal_mode_choices = ['train', 'meta', 'single']
+    annotate_parser.add_argument('--prodigal_mode', type=str, default='meta', choices=prodigal_mode_choices,
+                                 help='Mode of prodigal to use for gene calling. NOTE: normal or single mode require '
+                                      'genomes which are high quality with low contamination and long contigs (average '
+                                      'length >3 Kbp).')
+    # prodigal_trans_table_choices = ['auto'] + [str(i) for i in range(1, 26)]
+    prodigal_trans_table_choices = [str(i) for i in range(1, 26)]
+    annotate_parser.add_argument('--trans_table', type=str, default='11', choices=prodigal_trans_table_choices,
+                                 help='Translation table for prodigal to use for gene calling.')
     annotate_parser.add_argument('--bit_score_threshold', type=int, default=60,
                                  help='minimum bitScore of search to retain hits')
     annotate_parser.add_argument('--rbh_bit_score_threshold', type=int, default=350,
@@ -77,7 +89,7 @@ if __name__ == '__main__':
     annotate_genes_parser.add_argument('--keep_tmp_dir', action='store_true', default=False)
     annotate_genes_parser.add_argument('--threads', type=int, default=10, help='number of processors to use')
     annotate_genes_parser.add_argument('--verbose', action='store_true', default=False)
-    annotate_genes_parser.set_defaults(func=annotate_called_genes)
+    annotate_genes_parser.set_defaults(func=annotate_called_genes_cmd)
 
     # parser for summarizing genomes
     distill_parser.add_argument("-i", "--input_file", help="Annotations path")
@@ -86,6 +98,13 @@ if __name__ == '__main__':
     distill_parser.add_argument("--trna_path", help="tRNA output from annotation")
     distill_parser.add_argument("--groupby_column", help="Column from annotations to group as organism units",
                                 default='fasta')
+    distill_parser.add_argument("--custom_distillate", help="Custom distillate form to add your own modules")
+    distill_parser.add_argument("--distillate_gene_names", action='store_true', default=False,
+                                help="Give names of genes instead of counts in genome metabolism summary")
+    distill_parser.add_argument("--genomes_per_product", help="Number of genomes per product.html output. Decrease "
+                                                              "value if getting JavaScript Error: Maximum call stack "
+                                                              "size exceeded when viewing product.html in browser.",
+                                default=1000, type=int)
     distill_parser.set_defaults(func=summarize_genomes)
 
     # parser for getting genes
@@ -112,6 +131,23 @@ if __name__ == '__main__':
                             help='Maximum contamination of genome to keep genes')
     strainer_parser.set_defaults(func=pull_sequences)
 
+    # parser for getting gene neighborhoods
+    neighborhood_parser.add_argument("-i", "--input_file", help="Annotations path")
+    neighborhood_parser.add_argument("-o", "--output_dir", help="Directory to write gene neighborhoods")
+    neighborhood_parser.add_argument("--genes", nargs='*', help="Gene names from DRAM to find neighborhoods around")
+    neighborhood_parser.add_argument("--identifiers", nargs='*',
+                                     help="Database identifiers assigned by DRAM to find neighborhoods around")
+    neighborhood_parser.add_argument("--categories", help="Distillate categories to build gene neighborhoods around.")
+    neighborhood_parser.add_argument("--genes_loc", help="Location of genes.fna/genes.faa file to filter to "
+                                                         "neighborhoods")
+    neighborhood_parser.add_argument("--scaffolds_loc", help="Location of scaffolds.fna file to filter to "
+                                                             "neighborhoods")
+    neighborhood_parser.add_argument("--distance_genes", type=int, help="Number of genes away from center to include "
+                                                                        "in neighborhoods")
+    neighborhood_parser.add_argument("--distance_bp", type=int, help="Number of genes away from center to include "
+                                                                     "in neighborhoods")
+    neighborhood_parser.set_defaults(func=get_gene_neighborhoods)
+
     args = parser.parse_args()
     args_dict = {i: j for i, j in vars(args).items() if i != 'func'}
     args.func(**args_dict)
